/*
Copyright (C) 2006 Samuel Gravan Perez (samiloes@yahoo.es)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _SERVIDORSAM_H
#define _SERVIDORSAM_H

#define MAX_CON_IP 5000
#define MAX_USUARIOS 5000

#include <TcpSocket.h>
#include <IEventOwner.h>
#include <EventHandler.h>
#include <string>
#include <list>
#include <ext/hash_map>
#include <algorithm>


using namespace std;
using namespace __gnu_cxx;

struct Canal;

struct Cliente
{
	string nick;
	string ip;
	bool ping;
	int act;
	bool adm;
	list<Canal>::iterator canal;
	
	bool operator==(const string& c)
	{
	     if(c==this->nick)
	     	return true;
	     else
	     	return false;
	}	                                                                                 

};

struct Canal
{
	string nombre;
	list<Cliente> usuarios;
	
	bool operator==(const string& nombre)
	{
	    if(nombre==this->nombre)
	    	return true;
	    else
	    	return false;
	}
};

class MyEventHandler:public EventHandler
{
public:
	MyEventHandler(void):EventHandler()
	{
	}
	~MyEventHandler(void)
	{
	}
};

class stringhasher
{
public:
	size_t operator() (const string& str) const
	{
		unsigned int hash = 0;

		for(std::size_t i = 0; i < str.length(); i++)
		{
			hash ^= ((i & 1) == 0) ? (  (hash <<  7) ^ str[i] ^ (hash >> 3)) :
                               (~((hash << 11) ^ str[i] ^ (hash >> 5)));
		}

		return (hash & 0x7FFFFFFF);
	}

	bool operator() (const string& s1, const string& s2) const
	{
		return s1 < s2;
	}
};

class ServidorSAM : public TcpSocket,public IEventOwner
{
private:
	static list<Cliente> clientes;
	static list<Canal> canales;
	static hash_map<string,ServidorSAM*, stringhasher> clt_socket;
	static hash_map<string,int,stringhasher> ips;
	static list<string> baneados;
	static string estado;
	static int puerto;	
	static string pass;
	static string ipEscucha;
	Cliente c;
public:
	ServidorSAM(SocketHandler&);
	~ServidorSAM();

	void OnLine(const string&);
	void OnDelete(void);
	void OnConnectFailed(void);
	void OnException(void);
	void OnEvent(int);

	static void SetEstado(string est){ estado=est; }
	static void SetPuerto(int p){ puerto=p; }
	static string GetEstado(void){ return estado; }
	static int GetPuerto(void){ return puerto; }
	static int GetClientes(void){ return (int)clientes.size(); }
	static int GetCanales(void){ return (int)canales.size(); }
	static void SetPass(string p){ pass=p; }
	static string GetPass(void){ return pass; }
	static void SetIpEscucha(string i){ ipEscucha=i; }
	static string GetIpEscucha(void){ return ipEscucha; }
	static list<string> GetBaneados(void) {  return baneados; }
	static void PonerBan(string b) { baneados.push_back(b); }
	static void QuitarBan(string b) 
	{
		list<string>::iterator r=find(baneados.begin(),baneados.end(),b);
		if(r!=baneados.end())
			baneados.erase(r);	
	}
};

#endif
