/*
Copyright (C) 2006 Samuel Gravan Perez (samiloes@yahoo.es)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "ServidorSAM.h"
#include <ListenSocket.h>
#include <iostream>
#include <sstream>
#include <string>
#include <stdio.h>
#include <pthread.h>
#include <time.h>

using namespace std;

void Comando(void);
void* SelectBucle(void*);

MyEventHandler h;
ListenSocket<ServidorSAM> l(h);

int main(int argc, char* argv[])
{
	int puerto=6667;
	string pass="123",ip="127.0.0.1";
	ServidorSAM::SetEstado("ACTIVO");

	if(argc>1)
	{
		for(int i=1;i<argc;i++)
		{
			string opc(argv[i]);                		
			if(opc=="-p")									
				puerto=atoi(argv[++i]);
			else if(opc=="-s")
				pass=string(argv[++i]);
			else if(opc=="-i")
				ip=string(argv[++i]);
			else if(opc=="-e")
			{
				if(string(argv[i+1])=="ACTIVO" || string(argv[i+1])=="PARADO")
					ServidorSAM::SetEstado(string(argv[++i]));
			}
			else if(opc=="-a")
			{					
				cout << endl << "Opciones del programa:" << endl;
				cout << "-p para seleccionar el puerto donde arrancar" << endl;
				cout << "-i para seleccionar la direccion IP desde la que escuchar clientes" << endl;
				cout << "-s para seleccionar el password de administracion" << endl;
				cout << "-e para arrancar con un estado determinado [ACTIVO|PARADO]" << endl <<endl;
				exit(EXIT_SUCCESS);							
			}
		}
	}

    	ServidorSAM::SetPuerto(puerto);
    	ServidorSAM::SetPass(pass);
	ServidorSAM::SetIpEscucha(ip);
				
	if (l.Bind(ServidorSAM::GetIpEscucha(),ServidorSAM::GetPuerto(),30))
	{
		cout << "Error al abrir la escucha principal!!" << endl;
		cout << "Puerto: " << ServidorSAM::GetPuerto() << endl;
		exit(EXIT_FAILURE);
	}

	h.Add(&l);	
	h.Select(1,0);
	Comando();

	return EXIT_SUCCESS;
}

void* SelectBucle(void*)
{		
	while(1)
	{
		h.Select(1,0);
		h.CheckEvents();
	}
	return 0;
}

void Comando(void)
{
	string cmd;
	pthread_t slcthread;
	bool q=true;

	if(ServidorSAM::GetEstado()=="ACTIVO")
	{
		int p=pthread_create(&slcthread,NULL,SelectBucle,NULL);
		if(p)
		{
			cout << "Error al crear el hilo de eventos" << endl;
			exit(EXIT_FAILURE);
		}		
	}
	 
	while(q)
	{
		cout << "Servidor " << ServidorSAM::GetEstado() << ">";
		getline(cin,cmd);
		cmd=cmd.substr(0,20);
		if(cmd=="quit" || cmd=="q")
		{
			l.Close();
                        pthread_cancel(slcthread);                        
			//h.RemoveAllSockets();		
			q=false;
		}
		else 
		if(cmd=="estadistica" || cmd=="est")
		{
			list<string> b=ServidorSAM::GetBaneados();
			cout << "\nServidor escuchando en " << ServidorSAM::GetIpEscucha() << endl;
			cout << "Utilizando el puerto " << ServidorSAM::GetPuerto() << endl;
			cout << "Estado " << ServidorSAM::GetEstado() << endl;
			cout << "Usuarios activos:" << ServidorSAM::GetClientes() << endl;
			cout << "Canales activos:" << ServidorSAM::GetCanales() << endl; 
			cout << "Maximos usuarios permitidos por IP real:" << MAX_CON_IP << endl;
 			cout << "Password de administracion:" << ServidorSAM::GetPass() << endl << endl;
			if(b.size()>0)
			{
				cout << "Ips baneadas:";
				for(list<string>::iterator i=b.begin(); i!=b.end(); i++)
					cout << *i << endl;
			}
		}else
		if(cmd=="parar" || cmd=="p")
		{	
		    if(ServidorSAM::GetEstado()=="ACTIVO")
		    {	
			ServidorSAM::SetEstado("PARADO");
			pthread_cancel(slcthread);	
			l.Close();			
		    }
		}else
		if(cmd=="activar" || cmd=="a")
		{
		    if(ServidorSAM::GetEstado()=="PARADO")
		    {			
			if (l.Bind(ServidorSAM::GetIpEscucha(),ServidorSAM::GetPuerto(),30))
			{
				cout << "Error al abrir la escucha principal!!" << endl;
				cout << "Puerto: " << ServidorSAM::GetPuerto() << endl;
				exit(EXIT_FAILURE);
			}			
			int p=pthread_create(&slcthread,NULL,SelectBucle,NULL);
                	if(p)
			{
                        	cout << "Error al crear el hilo de eventos" << endl;
                        	exit(EXIT_FAILURE);
                	}			
			ServidorSAM::SetEstado("ACTIVO");
		    }
		}else
		if(cmd=="puerto" || cmd=="pu")
		{		
			cout << "Introduce el nuevo puerto:";
			getline(cin,cmd);
			if(cmd.length()>0)			
				ServidorSAM::SetPuerto(atoi(cmd.c_str()));			
		}else
                if(cmd=="pass" || cmd=="pa")
                {
                        cout << "Introduce el nuevo password del administrador:";
                        getline(cin,cmd);
                        if(cmd.length()>0)
                                ServidorSAM::SetPass(cmd);
                }
		else
		if(cmd=="ip" || cmd=="i")
		{
                        cout << "Introduce la direccion IP de escucha:";
                        getline(cin,cmd);
                        if(cmd.length()>0)
                                ServidorSAM::SetIpEscucha(cmd);
		}else
        	if(cmd=="qb" || cmd=="desbanear" || cmd=="quitar ban")
        	{
                	cout << "Introduce la ip para quitar el ban:";
                	getline(cin,cmd);
                	if(cmd.length()>0)
                      		ServidorSAM::QuitarBan(cmd);
        	}else
        	if(cmd=="pb" || cmd=="banear" || cmd=="poner ban")
        	{
                	cout << "Introduce la ip a banear:";
                	getline(cin,cmd);
                	if(cmd.length()>0)
                      		ServidorSAM::PonerBan(cmd);
		}
		else if(cmd=="ayuda" || cmd=="h")
		{
				cout << endl << "Comandos disponibles:" << endl << endl;
				cout << "(q)uit para salir del programa" << endl;
				cout << "(est)adistica para ver las estadisticas" << endl;
				cout << "(p)arar para detener el sistema" << endl;
				cout << "(a)activar para iniciar el servidor" << endl;
				cout << "(pu)erto para modificar el puerto de escucha" << endl;
				cout << "(pa)ss para modificar el password de administracion" << endl;
				cout << "(i)p para modificar la direccion de escucha de clientes" << endl;
				cout << "qb | desbanear | quitar ban para quitar un ban de usuario" << endl;
				cout << "pb | banear | poner ban para poner un ban a un usuario" << endl << endl;				
		}
		else if(cmd.length()>0)
			cout << "Comando desconocido!!" << endl;
	}	
}




